@@ -14,7 +14,7 @@ package MCE::Core::Validation;
 use strict;
 use warnings;
 
-our $VERSION = '1.602';
+our $VERSION = '1.603';
 
 ## Items below are folded into MCE.
 
@@ -36,8 +36,6 @@ sub _validate_args {
 
    @_ = ();
 
-   die 'Private method called' unless (caller)[0]->isa( ref $_s );
-
    my $_tag = 'MCE::_validate_args';
 
    if (defined $_s->{input_data} && ref $_s->{input_data} eq '') {
@@ -107,8 +105,6 @@ sub _validate_args_s {
 
    @_ = ();
 
-   die 'Private method called' unless (caller)[0]->isa( ref $self );
-
    my $_tag = 'MCE::_validate_args_s';
 
    if (defined $_s->{max_workers}) {
@@ -230,8 +226,6 @@ sub _validate_runstate {
 
    @_ = ();
 
-   die 'Private method called' unless (caller)[0]->isa( ref $self );
-
    _croak("$_tag: method cannot be called by the worker process")
       if ($self->{_wid});
    _croak("$_tag: method cannot be called while processing")
